
#define _GNU_SOURCE
#include <sched.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <syscall.h>

int main(int argc, char **argv)
{
    cpu_set_t cpuset, cpuget;
    char *program_name = *argv;
    int i = 0;
    int cpu_num = -1;
    pid_t pid = -1;

    while ((i = getopt(argc, argv, "c:p:?")) != EOF)
    {
        switch (i)
        {
        case 'c': /* 要绑定的cpu号*/
            cpu_num = atoi(optarg);
            break;
        case 'p': /* 要迁移的进程*/
            pid = atoi(optarg);
            break;
        case '?': // 命令行参数不正确，退出程序
            fprintf(stderr, "\nerr:cmd %s param is incorrect!\n", program_name);
            exit(2);
            break;
        }
    }

    if (argc <= 1 || cpu_num == -1 || pid == -1)
    {
        // 命令行参数不正确，退出程序
        fprintf(stderr, "\nerr:cmd %s param is incorrect!\n", program_name);
        exit(2);
    }

    CPU_ZERO(&cpuset);         // 初始化CPU集合，将 cpuset 置为空
    CPU_SET(cpu_num, &cpuset); // 将本进程绑定到 指定CPU 上
    // 设置进程的 CPU 亲和性
    if (sched_setaffinity(pid, sizeof(cpuset), &cpuset) == -1)
    {
        printf("Fail:can not switch thread %d to CPU %d\n", pid, cpu_num);
        return -1;
    }

    if (sched_getaffinity(pid, sizeof(cpuget), &cpuget) == -1)
    {
        printf("Fail:Get thread %d on CPU %d affinity failed, error: %s\n", pid, cpu_num, strerror(errno));
        return -1;
    }

    if (memcmp(&cpuset, &cpuget, sizeof(cpuset)) == 0)
    {
        printf("Pass: switch thread %d to cpu %d success\n", pid, cpu_num);
    }
    else
    {
        printf("Fail:can not switch thread %d to CPU %d\n", pid, cpu_num);
        return -1;
    }
    return 0;
}